/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: NPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla Communicator client code.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Roland Mainz <roland.mainz@informatik.med.uni-giessen.de>
 *   Micromuse Ltd
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the NPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/**	Code lifted from mozilla/xpfe/bootstrap and modified to:
	- Remove security contrains as ncisetup is only aimed to be a native application.
	- Relocated the storage of the user profile to a folder that do not conflict with mozilla itself.
	- Default to chrome ncisetup.jar
	- Allow a console mode that run like xpcshell and default to ncisetup.js

*/

#if 0
#include "nsXPCOMGlue.h"
#endif

#include "nsCOMPtr.h"
#include "XULDlgApp.h"
#include "nsCRT.h"

#include "nsIEventQueueService.h"

#ifdef NS_TRACE_MALLOC
#include "nsTraceMalloc.h"
#endif

#if defined(DEBUG) && defined(XP_WIN32)
#include <malloc.h>
#endif

#if defined (XP_MACOSX)
#include <Processes.h>
#endif

#if defined(DEBUG_pra)
#define DEBUG_CMD_LINE
#endif


extern "C" void ShowOSAlert(const char* aMessage);

#define HELP_SPACER_1   "\t"
#define HELP_SPACER_2   "\t\t"
#define HELP_SPACER_4   "\t\t\t\t"

#ifdef DEBUG
#include "prlog.h"
#endif

#ifdef MOZ_JPROF
#include "jprof.h"
#endif


#if defined(XP_UNIX) || defined(XP_BEOS)
  extern void InstallUnixSignalHandlers(const char *ProgramName);
#endif


#ifdef DEBUG_warren
#ifdef XP_WIN
#define _CRTDBG_MAP_ALLOC
#include <crtdbg.h>
#endif
#endif



int main(int argc, char* argv[])
{
	bool bDoGUI = true;
	nsCAutoString script;

	int rc=0;

	nsCAutoString chrome;



#if defined(DEBUG) && defined(XP_WIN32)
  // Disable small heap allocator to get heapwalk() giving us
  // accurate heap numbers. Win2k non-debug does not use small heap allocator.
  // Win2k debug seems to be still using it.
  // http://msdn.microsoft.com/library/default.asp?url=/library/en-us/vclib/html/_crt__set_sbh_threshold.asp
  _set_sbh_threshold(0);
#endif

#if defined(XP_UNIX) || defined(XP_BEOS)
  InstallUnixSignalHandlers(argv[0]);
#endif


	/// set the default mode...
#if defined(HPUX11) || defined(AIX5)
	bDoGUI = false;
	script = NS_LITERAL_CSTRING("chrome://ncisetup/content/console.js");
#else
	bDoGUI = true;
#endif
	chrome = NS_LITERAL_CSTRING("chrome://ncisetup/content/ncisetup.xul");

	for (int i=0 ; i < argc ; i++ ) {

		if ( strcmp( argv[i] , "-silent" ) == 0 ) {
			bDoGUI = false;
			script = NS_LITERAL_CSTRING("chrome://ncisetup/content/silent.js");
			break;
		}

		if ( strcmp( argv[i] , "-console" ) == 0 ) {
			bDoGUI = false;
			script = NS_LITERAL_CSTRING("chrome://ncisetup/content/console.js");
			break;
		}

		if ( strcmp( argv[i] , "-report" ) == 0 ) {
			bDoGUI = false;
			script = NS_LITERAL_CSTRING("chrome://ncisetup/content/report.js");
			break;
		}

		if ( strcmp( argv[i] , "-script" ) == 0 ) {
			if ( i+1 < argc ) {
				if ( argv[i+1] ) {
					bDoGUI = false;
					script = nsDependentCString(argv[i+1]);
					break;
				}
			}
		}

		if ( strcmp( argv[i] , "-chrome" ) == 0 ) {
			if ( i+1 < argc ) {
				if ( argv[i+1] ) {
					bDoGUI = true;
					chrome = nsDependentCString(argv[i+1]);
					break;
				}
			}
		}

	}

	CXULDlgApp dlg;

	dlg.AddRef();
	dlg.Initialize( argc ,argv, bDoGUI );

	if ( bDoGUI ) {
		rc = dlg.DoModal( chrome );
	} else {
		rc = dlg.DoScript( script );
	}

	dlg.Terminate();

	return rc;
}
