/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla Communicator client code, released
 * March 31, 1998.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   John Bandhauer <jband@netscape.com>
 *   Pierre Phaneuf <pp@ludusdesign.com>
 *	 Micromuse Ltd
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/**	Code lifted from mozilla/xpfe/bootstrap and modified to:
	- Remove security contrains as ncisetup is only aimed to be a native application.
	- Relocated the storage of the user profile to a folder that do not conflict with mozilla itself.
	- Default to chrome ncisetup.jar
	- Allow a console mode that run like xpcshell and default to ncisetup.js

*/

#include "XULDlgApp.h"

#include "nsXPCOM.h"
#if 0
#include "nsXPCOMGlue.h"
#endif
#include "nsProcess.h"
#include "nsCustomAppFileLocationProvider.h"

#include "nsIInterfaceRequestor.h"
#include "nsIInterfaceRequestorUtils.h"

#include "nsIWindowWatcher.h"
#include "nsIServiceManager.h"
#include "nsICmdLineService.h"
#include "nsIEventQueueService.h"


#include "nsIAppShell.h"
#include "nsAppShellCIDs.h"
#include "nsIAppShellService.h"

#include "nsIPref.h"

#include "nsIXULWindow.h"
#include "nsIIOService.h"
#include "nsIURI.h"
#include "nsIWebBrowserChrome.h"

#if defined(MOZ_WIDGET_GTK) || defined(MOZ_WIDGET_GTK2)
#include <gtk/gtk.h>
#endif //MOZ_WIDGET_GTK || MOZ_WIDGET_GTK2


#include "nsILocalFile.h"
#include "nsIChromeRegistry.h" // chromeReg

// #include "nsIChromeRegistry.h"
#include "nsILocaleService.h"

static NS_DEFINE_CID(kAppShellServiceCID,   NS_APPSHELL_SERVICE_CID);
static NS_DEFINE_CID(kCmdLineServiceCID,    NS_COMMANDLINE_SERVICE_CID);
static NS_DEFINE_CID(kIProcessCID,			NS_PROCESS_CID);
static NS_DEFINE_CID(kPrefCID, 				NS_PREF_CID);
static NS_DEFINE_CID(kEventQueueServiceCID, NS_EVENTQUEUESERVICE_CID);


CXULDlgApp::CXULDlgApp() : mEventQ( nsnull)
{
}

CXULDlgApp::~CXULDlgApp()
{
}


NS_IMPL_ISUPPORTS2(CXULDlgApp, nsIWindowCreator, nsIWindowCreator2)

/**	Most of this code is lifter from mozilla bootstrap itself...
*/
NS_IMETHODIMP CXULDlgApp::CreateChromeWindow(nsIWebBrowserChrome *aParent,
									 		 PRUint32 aChromeFlags,
											 nsIWebBrowserChrome **_retval)
{
	PRBool cancel;
	return CreateChromeWindow2(aParent, aChromeFlags, 0, 0, &cancel, _retval);
}

/** Most of this code is lifter from mozilla bootstrap itself...
*/
NS_IMETHODIMP CXULDlgApp::CreateChromeWindow2( nsIWebBrowserChrome *aParent,
											   PRUint32 aChromeFlags,
											   PRUint32 aContextFlags,
											   nsIURI *aURI,
											   PRBool *aCancel,
											   nsIWebBrowserChrome **_retval)
{
	NS_ENSURE_ARG_POINTER(aCancel);
	NS_ENSURE_ARG_POINTER(_retval);
	*aCancel = PR_FALSE;
	*_retval = 0;

	nsCOMPtr<nsIXULWindow> newWindow;

	if (aParent) {
		nsCOMPtr<nsIXULWindow> xulParent(do_GetInterface(aParent));
		NS_ASSERTION(xulParent, "window created using non-XUL parent. that's unexpected, but may work.");

		if (xulParent) {
			xulParent->CreateNewWindow(aChromeFlags, getter_AddRefs(newWindow));
		}
	} else {
		if (aChromeFlags & nsIWebBrowserChrome::CHROME_DEPENDENT) {
			NS_WARNING("dependent window created without a parent");
		}

		nsCOMPtr<nsIAppShellService> appShell(do_GetService(kAppShellServiceCID));
		if (!appShell) {
			return NS_ERROR_FAILURE;
		}

		appShell->CreateTopLevelWindow(	0,
										0,
										PR_FALSE,
										PR_FALSE,
										aChromeFlags,
										nsIAppShellService::SIZE_TO_CONTENT,
										nsIAppShellService::SIZE_TO_CONTENT,
										getter_AddRefs(newWindow));
	}

	// if anybody gave us anything to work with, use it
	if (newWindow) {
		newWindow->SetContextFlags(aContextFlags);
		nsCOMPtr<nsIInterfaceRequestor> req(do_QueryInterface(newWindow));
		if (req) {
			req->GetInterface(NS_GET_IID(nsIWebBrowserChrome), (void **) _retval);
		}
	}

	return *_retval ? NS_OK : NS_ERROR_FAILURE;
}

/**	Most of this code is lifter from the embedding test app...
*/
nsresult CXULDlgApp::InitializeWindowCreator()
{
	// give an nsIWindowCreator to the WindowWatcher service
	nsCOMPtr<nsIWindowCreator> windowCreator(NS_STATIC_CAST(nsIWindowCreator *, this));
	if (windowCreator) {
		nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService(NS_WINDOWWATCHER_CONTRACTID));
		if (wwatch) {
			wwatch->SetWindowCreator(windowCreator);
			return NS_OK;
		}
	}
	return NS_ERROR_FAILURE;
}


/**	This is probably useless and could be removed.
*/
nsresult CXULDlgApp::TerminateWindowCreator()
{
	// give an nsIWindowCreator to the WindowWatcher service
	nsCOMPtr<nsIWindowWatcher> wwatch(do_GetService(NS_WINDOWWATCHER_CONTRACTID));
	if (wwatch) {
		wwatch->SetWindowCreator(NULL);
		return NS_OK;
	}
  	return NS_ERROR_FAILURE;
}


/**	Initialize XPCOM and all the stuff that will be required by the XUL
	dialog app...
*/
nsresult CXULDlgApp::Initialize( int argc, char* argv[], bool bDoGUI )
{
#if 0
	/// @todo: replace the NULL by a fixed location such as $NCHOME/install
	nsresult rv = XPCOMGlueStartup( NULL );

	if (NS_FAILED(rv)) {
		NS_WARNING("gre: XPCOMGlueStartup failed");
		return rv;
	}
#else
	nsresult rv;
#endif

#if defined(MOZ_WIDGET_GTK) || defined(MOZ_WIDGET_GTK2)
	// We'd normally check against bDoGUI here, but we mozilla doesn't
	// like working without X:(
	if (bDoGUI) {
		// Initialize GTK+1/2 here for splash
#if defined(MOZ_WIDGET_GTK)
		gtk_set_locale();
#endif
		gtk_init(&argc, &argv);

		gtk_widget_set_default_visual(gdk_rgb_get_visual());
		gtk_widget_set_default_colormap(gdk_rgb_get_cmap());
	}
#endif /* MOZ_WIDGET_GTK || MOZ_WIDGET_GTK2 */



	/// Implement a folder provider. This is mostly to overwrite the cache folder that is created in the user directory
	nsCustomAppFileLocationProvider *provider = new nsCustomAppFileLocationProvider();
	rv = NS_InitXPCOM2(nsnull, nsnull, provider);
	if (NS_FAILED(rv)) {
		// We should be displaying a dialog here with the reason why we failed.
		NS_WARNING("NS_InitXPCOM2 failed");
		return 1;
	}

	/// Initialize a Message Queue (just in cas)
	nsCOMPtr<nsIEventQueueService> eventQService = do_GetService(kEventQueueServiceCID, &rv);
	if (NS_FAILED(rv))
		return rv;

	rv = eventQService->CreateMonitoredThreadEventQueue();
	if (NS_FAILED(rv))
		return rv;

	rv = eventQService->GetThreadEventQueue(NS_CURRENT_THREAD, &mEventQ);
	if (NS_FAILED(rv))
		return rv;



	/// Initialize the cmd line service
	nsCOMPtr<nsICmdLineService> cmdLineArgs(do_GetService(kCmdLineServiceCID, &rv));
	NS_ASSERTION(NS_SUCCEEDED(rv), "Could not obtain CmdLine processing service\n");
	if (NS_FAILED(rv))
		return rv;

	rv = cmdLineArgs->Initialize(argc, argv);
	NS_ASSERTION(NS_SUCCEEDED(rv), "failed to initialize command line args");
	if (rv == NS_ERROR_INVALID_ARG) {
		return rv;
	}

	if (bDoGUI) {
		nsCOMPtr<nsIAppShellService> appShell(do_GetService(kAppShellServiceCID, &rv));
		NS_ASSERTION(NS_SUCCEEDED(rv), "failed to get the appshell service");
		if (NS_FAILED(rv))
			return rv;

		rv = appShell->Initialize(cmdLineArgs, appShell);
	}

	{ /// relax the security
	    nsCOMPtr<nsIPref> mPref;

		mPref = do_GetService(kPrefCID, &rv);

	    mPref->SetBoolPref("signed.applets.codebase_principal_support" , PR_TRUE);
	    mPref->SetBoolPref("security.xpconnect.plugin.unrestricted" , PR_TRUE);
	    mPref->SetBoolPref("security.checkloaduri" , PR_FALSE);
	    mPref->SetBoolPref("javascript.enabled" , PR_TRUE );

		mPref->SetCharPref( "capability.principal.codebase.p0.granted" , "UniversalXPConnect UniversalBrowserWrite UniversalBrowserRead");
		mPref->SetCharPref( "capability.principal.codebase.p0.id" , "file://");
		mPref->SetCharPref( "capability.principal.codebase.p1.granted" , "UniversalXPConnect UniversalBrowserWrite UniversalBrowserRead");
		mPref->SetCharPref( "capability.principal.codebase.p1.id" , "http://");
		mPref->SetCharPref( "capability.principal.codebase.p2.granted" , "UniversalXPConnect UniversalBrowserWrite UniversalBrowserRead");
		mPref->SetCharPref( "capability.principal.codebase.p2.id" , "https://");



/*		{
		}
*/
		{	/** Set the locale ... */
			/* May want an overriding value that is stored in the Install Database (SQLite) */

			/* This code reads from the command line first, then reads from the system locale */
			/* if no command line option was set */

			char * varstr = NULL;
			rv = cmdLineArgs->GetCmdLineValue("-UILocale", &varstr);
			if (NS_SUCCEEDED(rv))
			{
				nsCOMPtr<nsIXULChromeRegistry> chromeRegistry = do_GetService(NS_CHROMEREGISTRY_CONTRACTID, &rv);
				if (chromeRegistry)
				{
					if (varstr)
					{
						nsCAutoString UILocaleName(varstr);
						rv = chromeRegistry->SelectLocale(nsDependentCString(varstr), PR_FALSE);
						if ( !NS_SUCCEEDED(rv) ) {
							fprintf( stderr, "SelectLocale failed: rv = 0x%08x\n", rv);
						}
						mPref->SetCharPref( "ncisetup.locale", UILocaleName.get() );

					}
					else
					{
						nsAutoString appLocaleVal;
						nsCOMPtr<nsILocaleService> localeService = do_GetService(NS_LOCALESERVICE_CONTRACTID, &rv);

						rv = localeService->GetLocaleComponentForUserAgent(appLocaleVal);
						if(NS_SUCCEEDED(rv))
						{
							NS_ConvertUCS2toUTF8 appLocaleCVal(appLocaleVal);
							rv = chromeRegistry->SelectLocale(appLocaleCVal, PR_FALSE);
							mPref->SetCharPref( "ncisetup.locale", appLocaleCVal.get() );
						}
					}
				}
			}
		}
	}
	/// Initialize the Window Creator... This is required for alert() is JScript later on from the XUL world...
	InitializeWindowCreator();


	return rv;
}



/**	Display the main dialog itself.
*/
nsresult CXULDlgApp::DoModal(const nsACString &url) {

	nsresult rv = NS_OK;

	nsCOMPtr<nsIAppShellService> appShell(do_GetService(kAppShellServiceCID, &rv));
	NS_ASSERTION(NS_SUCCEEDED(rv), "failed to get the appshell service");

	nsCOMPtr<nsIXULWindow> newWindow;

	// Get a pointer to the IOService, this only to create a URL...
	nsCOMPtr<nsIIOService> mIOService( do_GetService("@mozilla.org/network/io-service;1", &rv));

	nsCOMPtr<nsIURI> uri;
	// nsEmbedCString urlString(node->urlString);
	mIOService->NewURI(url, nsnull,  nsnull,  getter_AddRefs(uri));

	rv = appShell->CreateTopLevelWindow(0,
										uri,
										PR_TRUE,
										PR_TRUE,
										1,
										nsIAppShellService::SIZE_TO_CONTENT,
										nsIAppShellService::SIZE_TO_CONTENT,
										getter_AddRefs(newWindow));



	// Start main event loop
	rv = appShell->Run();

	NS_ASSERTION(NS_SUCCEEDED(rv), "failed to run appshell");

	return rv;
}


/** Attempts to close XPCOM nicelly
*/
nsresult CXULDlgApp::Terminate() {

	nsresult rv = NS_OK;

	TerminateWindowCreator();

	NS_RELEASE(mEventQ);
    PR_Sleep(PR_SecondsToInterval(1));

	rv = NS_ShutdownXPCOM(nsnull);
	NS_ASSERTION(NS_SUCCEEDED(rv), "NS_ShutdownXPCOM failed");
#if 0
	XPCOMGlueShutdown();
#endif

	return rv;
}

