/**
 * Pre-remove script to clean up Webtop after uninstallation.
 *
 * Note on exception handling:
 * All exceptions are caught and logged in this main function. Other functions
 * shouldn't be catching exceptions so that this function does not carry on
 * with the post-install actions if an error occurs (which otherwise could
 * mess things up).
 */
function preRemove()
{
	log( 2, "Starting webtop preRemove()..." );

	try
	{
		// --- Configure the uninstall-Ant script (for removing roles) ---
		var uninstallScript = getNchome() +
			"/etc/webtop/installer/machant_webtop_uninstall.xml.orig";

		log( 2, "Configuring " + uninstallScript );

		var uninstallScriptContents = readFromFile( uninstallScript );
		var newUninstallScriptContents =
			processUninstallAntScript( uninstallScriptContents );

		// NBS 45973: The processed uninstall script is not placed directly into the antonce
		// directory so that Webtop roles and groups are not removed in the case where there are
		// multiple Webtop/Webtop-dependent installations using the same Security Manager server.
		var newUninstallScript = getNchome() +
			"/etc/webtop/installer/machant_webtop_uninstall.xml";
		writeToFile( newUninstallScript , newUninstallScriptContents, false );

		log( 3, "machant_webtop_uninstall.xml configuration completed." );
		log( 1, "In order to remove Netcool/Webtop groups and roles, copy "
			+ transformPathSeparators( newUninstallScript ) + " to the "
			+ transformPathSeparators( getNchome() + "/etc/guifoundation/antonce" )
			+ " directory." );
	}
	catch ( e )
	{
		log( 0, e );
		log( 0, "webtop preRemove() failed to complete." );

		return;
	}

	log( 3, "webtop preRemove() completed successfully." );
}

/**
 * Process the content of the uninstallation Ant script.
 * <p>
 * The ant script contains the following variables:
 * <ul>
 * <li>__NCSM0_ADMIN_USERNAME__</li>
 * <li>__NCSM0_ADMIN_PASSWORD__</li>
 * <li>__GUIFOUNDATION__</li>
 * </ul>
 *
 * @param txt (String) The content to process.
 * @return (String) The processed content.
 */
function processUninstallAntScript( fileContents )
{
	log( 2, "Processing uninstall Ant script..." );

	var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].
		getService(Components.interfaces.nsINCIProperties);

	var mainHttpServerUsername =
		propMgr.GetProp("NCSM0.ADMIN.USERNAME").value;
	var mainHttpServerPassword =
		propMgr.GetProp("NCSM0.ADMIN.PASSWORD").value;
	var guifoundation = getNchome() + "/guifoundation";
	var mainHttpServerPort = propMgr.GetProp("GUIFOUNDATION.WEBSERVER.PORT").value;

	var newContents = fileContents.replace( /__GUIFOUNDATION__/g,
		guifoundation );

	newContents = newContents.replace( /__GUIFOUNDATION_WEBSERVER_PORT__/g, mainHttpServerPort );

	newContents = newContents.replace( /__NCSM0_ADMIN_USERNAME__/g,
		mainHttpServerUsername );

	newContents = newContents.replace( /__NCSM0_ADMIN_PASSWORD__/g,
		mainHttpServerPassword );

	log( 3, "Uninstall Ant script processing completed." );

	return newContents;
}
